/*
 * Copyright (c) 2008 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include <assert.h>
#include <pcap.h>
#include <stdio.h>
#include <string.h>

#ifdef HAVE_POCOLIBS
#include <portLib.h>
#endif
#include <velodyne/velodyneLib.h>

int
velodyne_pcap_file_open(char *name, void *pcap_ptr)
{
	pcap_t *pcap;
	char errbuf[PCAP_ERRBUF_SIZE];

	pcap = pcap_open_offline(name, errbuf);
	if (pcap == NULL) {
		fprintf(stderr, "pcap_open: %s\n",  errbuf);
#ifdef HAVE_POCOLIBS
		errnoSet(S_velodyneLib_FILE_OPEN_ERROR);
#endif
		return -1;
	}
	*(pcap_t **)pcap_ptr = pcap;
	return 0;
}

void
velodyne_close_pcap(void *pcap)
{
	pcap_close((pcap_t *)pcap);
}

int
velodyne_pcap_read_packet(void *pcap_ptr, VELODYNE_PACKET_STR *p)
{
	pcap_t *pcap = (pcap_t *)pcap_ptr;
	struct pcap_pkthdr *header;
	const u_char *data;

	switch(pcap_next_ex(pcap, &header, &data)) {
	case 1:
		/* OK */
		assert(header->len == VELODYNE_MSG_BUFFER_SIZE + 42);
		/* Skip packet headers
		 * 14 bytes ethernet
		 * 20 bytes IP (XXX can be variable in real life)
		 * 8 bytes UDP
		 * --> total 42 */
		memcpy(&p->data, data+42, VELODYNE_MSG_BUFFER_SIZE);
		memcpy(&p->ts, &header->ts, sizeof(struct timeval));
		return VELODYNE_MSG_BUFFER_SIZE;
	case 0:
		/* timeout - shouldn't happen on file */
		break;
	case -1:
#ifdef HAVE_POCOLIBS
		errnoSet(S_velodyneLib_READ_ERROR);
#endif
		return -1;
	case -2:
		/* end of file */
		return 0;
	}
}
